<?php

namespace Bixo\Map\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Map\Forms\Developer as DeveloperForm;
use Bixo\Map\Http\Requests\DeveloperResourceRequest;
use Bixo\Map\Http\Resources\DeveloperResource;
use Bixo\Map\Http\Resources\DevelopersCollection;
use Bixo\Map\Models\Developer;
use Bixo\Map\Actions\DeveloperAction;
use Bixo\Map\Actions\DeveloperActions;


/**
 * Resource controller class for developer.
 */
class DeveloperResourceController extends BaseController
{

    /**
     * Initialize developer resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = DeveloperForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.map.modules'), 'map', guard_url('map'));
            return $next($request);
        });
    }

    /**
     * Display a list of developer.
     *
     * @return Response
     */
    public function index(DeveloperResourceRequest $request)
    {
        $request = $request->all();
        $page = DeveloperActions::run('paginate', $request);

        $data = new DevelopersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('map::developer.names'))
            ->view('map::developer.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display developer.
     *
     * @param Request $request
     * @param Model   $developer
     *
     * @return Response
     */
    public function show(DeveloperResourceRequest $request, Developer $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DeveloperResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('map::developer.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('map::developer.show')
            ->output();
    }

    /**
     * Show the form for creating a new developer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DeveloperResourceRequest $request, Developer $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DeveloperResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('map::developer.name'))
            ->view('map::developer.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new developer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DeveloperResourceRequest $request, Developer $model)
    {
        try {
            $request = $request->all();
            $model = DeveloperAction::run('store', $model, $request);
            $data = new DeveloperResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('map::developer.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('map/developer/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/map/developer'))
                ->redirect();
        }

    }

    /**
     * Show developer for editing.
     *
     * @param Request $request
     * @param Model   $developer
     *
     * @return Response
     */
    public function edit(DeveloperResourceRequest $request, Developer $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DeveloperResource($model);
        // return view('map::developer.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('map::developer.name'))
            ->view('map::developer.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the developer.
     *
     * @param Request $request
     * @param Model   $developer
     *
     * @return Response
     */
    public function update(DeveloperResourceRequest $request, Developer $model)
    {
        try {
            $request = $request->all();
            $model = DeveloperAction::run('update', $model, $request);
            $data = new DeveloperResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('map::developer.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/developer/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/developer/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the developer.
     *
     * @param Model   $developer
     *
     * @return Response
     */
    public function destroy(DeveloperResourceRequest $request, Developer $model)
    {
        try {

            $request = $request->all();
            $model = DeveloperAction::run('destroy', $model, $request);
            $data = new DeveloperResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('map::developer.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/developer/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/developer/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
