<?php

namespace Bixo\Map\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Map\Forms\Project as ProjectForm;
use Bixo\Map\Http\Requests\ProjectResourceRequest;
use Bixo\Map\Http\Resources\ProjectResource;
use Bixo\Map\Http\Resources\ProjectsCollection;
use Bixo\Map\Models\Project;
use Bixo\Map\Actions\ProjectAction;
use Bixo\Map\Actions\ProjectActions;


/**
 * Resource controller class for project.
 */
class ProjectResourceController extends BaseController
{

    /**
     * Initialize project resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = ProjectForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.map.modules'), 'map', guard_url('map'));
            return $next($request);
        });
    }

    /**
     * Display a list of project.
     *
     * @return Response
     */
    public function index(ProjectResourceRequest $request)
    {
        $request = $request->all();
        $page = ProjectActions::run('paginate', $request);

        $data = new ProjectsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('map::project.names'))
            ->view('map::project.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display project.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function show(ProjectResourceRequest $request, Project $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProjectResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('map::project.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('map::project.show')
            ->output();
    }

    /**
     * Show the form for creating a new project.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ProjectResourceRequest $request, Project $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProjectResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('map::project.name'))
            ->view('map::project.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new project.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProjectResourceRequest $request, Project $model)
    {
        try {
            $request = $request->all();
            $model = ProjectAction::run('store', $model, $request);
            $data = new ProjectResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('map::project.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('map/project/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/map/project'))
                ->redirect();
        }

    }

    /**
     * Show project for editing.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function edit(ProjectResourceRequest $request, Project $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ProjectResource($model);
        // return view('map::project.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('map::project.name'))
            ->view('map::project.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the project.
     *
     * @param Request $request
     * @param Model   $project
     *
     * @return Response
     */
    public function update(ProjectResourceRequest $request, Project $model)
    {
        try {
            $request = $request->all();
            $model = ProjectAction::run('update', $model, $request);
            $data = new ProjectResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('map::project.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/project/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/project/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the project.
     *
     * @param Model   $project
     *
     * @return Response
     */
    public function destroy(ProjectResourceRequest $request, Project $model)
    {
        try {

            $request = $request->all();
            $model = ProjectAction::run('destroy', $model, $request);
            $data = new ProjectResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('map::project.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/project/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/project/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
