<?php

namespace Bixo\Map\Http\Controllers;

use Exception;
use Bixo\Map\Actions\ProjectWorkflow;
use Bixo\Map\Http\Requests\ProjectWorkflowRequest;
use Bixo\Map\Http\Resources\ProjectResource;
use Bixo\Map\Models\Project;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class ProjectWorkflowController extends BaseController
{
    /**
     * Action controller function for project.
     *
     * @param Model $project
     * @param action next action for the project.
     *
     * @return Response
     */
    public function __invoke(ProjectWorkflowRequest $request, Project $project, $trasition)
    {
        try {
            $request = $request->all();
            $project = ProjectWorkflow::run($trasition, $project, $request);
            $data = new ProjectResource($project);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('map::project.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('map/project/' . $project->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/project/' . $project->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the project and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $project
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $project, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $project = Project::findBySignedId($project);
        $transitions = $this->transitions($project, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('map::project.name') . ' ' . trans('Approvals'))
            ->view('map::public.project.workflow')
            ->layout('mobile')
            ->data(compact('project', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the project.
     *
     * @param PublicRequest $request
     * @param string $project
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $project, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $project = Project::findBySignedId($project);
        $user = user();
        $project = ProjectWorkflow::run($transition, $project, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($project, $user)
    {
        $transitions = [];
        foreach ($project->workflow()->transitions($project) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.project.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'project' => $project->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $project->workflow()->form($value);
            $array['label'] = trans('map::project.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
