<?php

namespace Bixo\Map\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Map\Forms\Transaction as TransactionForm;
use Bixo\Map\Http\Requests\TransactionResourceRequest;
use Bixo\Map\Http\Resources\TransactionResource;
use Bixo\Map\Http\Resources\TransactionsCollection;
use Bixo\Map\Models\Transaction;
use Bixo\Map\Actions\TransactionAction;
use Bixo\Map\Actions\TransactionActions;


/**
 * Resource controller class for transaction.
 */
class TransactionResourceController extends BaseController
{

    /**
     * Initialize transaction resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = TransactionForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.map.modules'), 'map', guard_url('map'));
            return $next($request);
        });
    }

    /**
     * Display a list of transaction.
     *
     * @return Response
     */
    public function index(TransactionResourceRequest $request)
    {
        $request = $request->all();
        $page = TransactionActions::run('paginate', $request);

        $data = new TransactionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('map::transaction.names'))
            ->view('map::transaction.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function show(TransactionResourceRequest $request, Transaction $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TransactionResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('map::transaction.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('map::transaction.show')
            ->output();
    }

    /**
     * Show the form for creating a new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TransactionResourceRequest $request, Transaction $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TransactionResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('map::transaction.name'))
            ->view('map::transaction.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new transaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TransactionResourceRequest $request, Transaction $model)
    {
        try {
            $request = $request->all();
            $model = TransactionAction::run('store', $model, $request);
            $data = new TransactionResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('map::transaction.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('map/transaction/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/map/transaction'))
                ->redirect();
        }

    }

    /**
     * Show transaction for editing.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function edit(TransactionResourceRequest $request, Transaction $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TransactionResource($model);
        // return view('map::transaction.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('map::transaction.name'))
            ->view('map::transaction.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the transaction.
     *
     * @param Request $request
     * @param Model   $transaction
     *
     * @return Response
     */
    public function update(TransactionResourceRequest $request, Transaction $model)
    {
        try {
            $request = $request->all();
            $model = TransactionAction::run('update', $model, $request);
            $data = new TransactionResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('map::transaction.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/transaction/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/transaction/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the transaction.
     *
     * @param Model   $transaction
     *
     * @return Response
     */
    public function destroy(TransactionResourceRequest $request, Transaction $model)
    {
        try {

            $request = $request->all();
            $model = TransactionAction::run('destroy', $model, $request);
            $data = new TransactionResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('map::transaction.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('map/transaction/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('map/transaction/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
